# -*- coding: utf-8 -*-

# Copyright (C) 2007 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""
Website URLs.

:var BUG_REPORT_URL: Web page where to submit bug reports
:var DOCUMENTATION_URL: Documentation for users
:var HOMEPAGE_URL: Gaupol web pages
:var REGEX_HELP_URL: Documentation on regular expressions
"""

__all__ = (
    "BUG_REPORT_URL",
    "DOCUMENTATION_URL",
    "HOMEPAGE_URL",
    "REGEX_HELP_URL",
)

BUG_REPORT_URL = "https://github.com/otsaloma/gaupol/issues"
DOCUMENTATION_URL = "https://github.com/otsaloma/gaupol/tree/master/doc"
HOMEPAGE_URL = "https://otsaloma.io/gaupol/"
REGEX_HELP_URL = "https://docs.python.org/3/library/re.html#regular-expression-syntax"
